# Generated by Django 3.1.2 on 2020-10-23 12:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0042_auto_20201015_1328'),
        ('geostore', '0042_layer_routable'),
    ]

    operations = [
    ]
