# Generated by Django 2.0.13 on 2019-05-03 15:26

from django.db import migrations, models
import geostore.models


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0022_auto_20190417_1610'),
    ]

    operations = [
        migrations.AddField(
            model_name='layer',
            name='geom_type',
            field=models.IntegerField(choices=[(geostore.GeometryTypes(0), 0), (geostore.GeometryTypes(1), 1), (geostore.GeometryTypes(3), 3), (geostore.GeometryTypes(4), 4), (geostore.GeometryTypes(5), 5), (geostore.GeometryTypes(6), 6), (geostore.GeometryTypes(7), 7)], null=True),
        ),
    ]
