# Generated by Django 3.1.2 on 2020-10-15 13:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0041_auto_20200625_1515'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='feature',
            constraint=models.CheckConstraint(check=models.Q(geom__isempty=False), name='geom_is_empty'),
        ),
    ]
